IRActorBehavior subclass: #Guards1ActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards'!


!Guards1ActorBehavior methodsFor: 'message handling'!

nextMessage
	"First naive implementation of guards:
	If a guard is false, resend the message and look for next one."	

	"Note that because of resending it does not ensure message ordering preservation."

	| message |
	message := super nextMessage.
	[self satisfyConstraintSelector: message selector arguments: message arguments]
		whileFalse:
			[aself resend: message.
			message := super nextMessage].
	^message! !

!Guards1ActorBehavior methodsFor: 'guard testing'!

findGuardSelector: selector
	"A guard is simply associated to a selector with the prefix guardOf.
	For instance the guard of method put: is the method guardOfPut:."

	^('guardOF' , selector) asSymbol!

satisfyConstraintSelector: selector arguments: argumentsArray
	"Simply evaluate the guard method whih is assumed to be boolean."

	^self perform: (self findGuardSelector: selector) withArguments: argumentsArray! !

Guards1ActorBehavior subclass: #GuardsActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards'!


!GuardsActorBehavior methodsFor: 'message handling'!

nextMessage
	"Look for the first message in the mailbox verifying its guard.
	If all current pending messages have false guards,
	then suspend until next received message and check its guard, etc...
	This makes use of method atIndex: which suspends until there is a ith message in the mailBox."

	"As opposed to the initial naive strategy of resending the message
	(class Guards1SynchroActorBehavior), this strategy is more efficient
	and it also ensures message ordering preservation."

	"About guards reevaluation semantics:
	Guards are reevaluated (only) when looking for a new message to accept.
	There is no intra-object concurrency in this model of guards."

	| i message |
	i := 1.
	message := aself mailBox atIndex: i.
	[self satisfyConstraintSelector: message selector arguments: message arguments]
		whileFalse:
			[i := i+1.
			message := aself mailBox atIndex: i].
	aself mailBox removeAtIndex: i.
	^message! !

GuardsActorBehavior subclass: #CheckGuardsActorBehavior
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-Guards'!


!CheckGuardsActorBehavior methodsFor: 'guard testing'!

findGuardSelector: selector

	^self class methodToGuardSelectorDictionary at: selector! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CheckGuardsActorBehavior class
	instanceVariableNames: 'methodToGuardSelectorDictionary '!


!CheckGuardsActorBehavior class methodsFor: 'initialize'!

initialize

	"Note that the guard selector dictionary is implemented as a metaclass instance variable
	because it should not be shared by subclasses."
	methodToGuardSelectorDictionary := IdentityDictionary new.
	self initializeAndCheckGuards!

initializeAndCheckGuards
	"Initialize the dictionary associating guard selectors with script selectors.
	Therefore chek if every script selector has a corresponding guard (possibly inherited).
	If a guard is missing, produce a warning and create a default constant (true) guard."

	| guardSelector |
	self allScriptSelectors do: [:selector |
		guardSelector := ('guardOF' , selector) asSymbol.
		"Check if guard is existing otherwise produce a warning and create a default true guard."
		(self canUnderstand: guardSelector)
			ifFalse:
				[Transcript show: 'Warning: method ' , selector asString
								, ' of class ' , self printString , ' has no guard defined.'; cr;
						show: 'I produce (always true) guard method ' , guardSelector asString; cr.
				"To retrieve the message pattern (including arguments),
				we make the assumption that there is a CR (13) after it."
				self compile: ('guardOF' , ((self sourceMethodAt: selector) copyUpTo: 13 asCharacter)

							, '\\	^true') withCRs
					classified: 'guards'].
		methodToGuardSelectorDictionary at: selector put: guardSelector]! !

!CheckGuardsActorBehavior class methodsFor: 'accessing'!

methodToGuardSelectorDictionary

	^methodToGuardSelectorDictionary! !
CheckGuardsActorBehavior initialize!

