StateActorBehavior subclass: #StateBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-State-Ex'!


!StateBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger

	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!StateBoundedBuffer methodsFor: 'abstract states'!

empty

	^#(put:)!

full

	^#(get)!

initialAbstractState

	^#empty!

partial
	"Note that we use union of sets (in fact concatenation of arrays) for better modularity.
	See class StateSynchroGet2BoundedBuffer redefinition of #full,
	and consequently implicit redefinition of #partial."

	^(self empty) + (self full)! !

!StateBoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector

	^contents isEmpty
		ifTrue: [#empty]
		ifFalse: [contents isFull
					ifTrue: [#full]
					ifFalse: [#partial]]! !

!StateBoundedBuffer methodsFor: 'script'!

get

	^contents removeFirst!

put: item

	contents addLast: item! !

!StateBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StateBoundedBuffer class
	instanceVariableNames: ''!


!StateBoundedBuffer class methodsFor: 'instance creation'!

new: size

	^self new initialize: size! !

!StateBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"StateBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 4"
	"StateBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (StateBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGet: numberItems! !

StateBoundedBuffer subclass: #StateGgetBoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-State-Ex'!


!StateGgetBoundedBuffer methodsFor: 'abstract states'!

partialNotAfterPut

	^(self partial) + #(gget)! !

!StateGgetBoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector
	"We check if the bounded buffer is in partial state AND not after a put."
	"Note that the test [contents isFull not] (to make sure that it is partial)
	may be omitted because we may assume that after a non put, the buffer should not be full."

	^(contents isEmpty not and: [selector ~= #put:])
		ifTrue: [#partialNotAfterPut]
		ifFalse: [super nextAbstractStateAfter: selector]! !

!StateGgetBoundedBuffer methodsFor: 'script'!

get

	^contents removeFirst!

gget

	^self get!

put: item

	contents addLast: item! !

!StateGgetBoundedBuffer methodsFor: 'events'!

eventAccept: aMessage

	Transcript show: self printString , ' accept (' , aMessage compactPrintString , ')'; cr!

eventReceive: aMessage

	Transcript show: self printString , ' receive (' , aMessage compactPrintString , ')'; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StateGgetBoundedBuffer class
	instanceVariableNames: ''!


!StateGgetBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"StateGgetBoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 1"

	| buffer producer consumer |
	buffer := (StateGgetBoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGget: numberItems! !

StateBoundedBuffer subclass: #StateGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Synchro-State-Ex'!


!StateGet2BoundedBuffer methodsFor: 'abstract states'!

full

	^(super full) + #(get2)!

one

	^(self partial) - #(get2)

	"Note that an alternative is: (self empty) + #(get)."! !

!StateGet2BoundedBuffer methodsFor: 'state transition'!

nextAbstractStateAfter: selector

	^contents size = 1
		ifTrue: [#one]
		ifFalse: [super nextAbstractStateAfter: selector]! !

!StateGet2BoundedBuffer methodsFor: 'script'!

get

	^contents removeFirst!

get2

	^Array
		with: contents removeFirst
		with: contents removeFirst!

put: item

	contents addLast: item! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StateGet2BoundedBuffer class
	instanceVariableNames: ''!


!StateGet2BoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"StateGet2BoundedBuffer exampleSize: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (StateGet2BoundedBuffer new: maxSize) actor.
	producer := (Producer new buffer: buffer delay: 1) actor.
	consumer := (Consumer new buffer: buffer delay: ratio) actor.
	producer runPut: numberItems.
	consumer runGet2: numberItems! !
